@@ -1503,23 +1503,24 @@ def detect_obsolete_substvars(
 
 def read_dh_addon_sequences(
     debian_dir: VirtualPath,
-) -> Optional[Tuple[Set[str], Set[str]]]:
+) -> Optional[Tuple[Set[str], Set[str], bool]]:
     ctrl_file = debian_dir.get("control")
     if ctrl_file:
         dr_sequences: Set[str] = set()
         bd_sequences: Set[str] = set()
 
         drules = debian_dir.get("rules")
+        saw_dh = False
         if drules and drules.is_file:
             with drules.open() as fd:
-                parse_drules_for_addons(fd, dr_sequences)
+                saw_dh = parse_drules_for_addons(fd, dr_sequences)
 
         with ctrl_file.open() as fd:
             ctrl = list(Deb822.iter_paragraphs(fd))
         source_paragraph = ctrl[0] if ctrl else {}
 
         extract_dh_addons_from_control(source_paragraph, bd_sequences)
-        return bd_sequences, dr_sequences
+        return bd_sequences, dr_sequences, saw_dh
     return None
 
 
@@ -1539,7 +1540,7 @@ def detect_dh_addons_zz_debputy_rrr(
         )
         return
 
-    bd_sequences, dr_sequences = r
+    bd_sequences, dr_sequences, _ = r
 
     remaining_sequences = bd_sequences | dr_sequences
     saw_dh_debputy = "zz-debputy-rrr" in remaining_sequences
@@ -1565,7 +1566,7 @@ def detect_dh_addons(
         )
         return
 
-    bd_sequences, dr_sequences = r
+    bd_sequences, dr_sequences, _ = r
 
     remaining_sequences = bd_sequences | dr_sequences
     saw_dh_debputy = (
