@@ -1194,7 +1194,10 @@ class HighLevelManifest:
                 for s in search_dirs
                 if s.search_dir.fs_path != source_root_dir.fs_path
             )
-            _present_installation_dirs(search_dirs, check_for_uninstalled_dirs, into)
+            if enable_manifest_installation_feature:
+                _present_installation_dirs(
+                    search_dirs, check_for_uninstalled_dirs, into
+                )
         else:
             dtmp_dir = None
             search_dirs = install_request_context.search_dirs
@@ -1404,7 +1407,8 @@ class HighLevelManifest:
                 dbgsym_info,
             )
 
-        _list_automatic_discard_rules(path_matcher)
+        if enable_manifest_installation_feature:
+            _list_automatic_discard_rules(path_matcher)
 
         return package_data_table
 
