@@ -6,7 +6,7 @@ import sys
 import textwrap
 from typing import Optional, List, Union, NoReturn, Mapping
 
-from lsprotocol.types import (
+from debputy.lsprotocol.types import (
     CodeAction,
     Command,
     CodeActionParams,
