@@ -1,4 +1,5 @@
 import dataclasses
+import os.path
 import re
 import textwrap
 from typing import (
@@ -9,30 +10,10 @@ from typing import (
     Mapping,
     List,
     Dict,
+    Iterable,
 )
 
-from lsprotocol.types import (
-    DiagnosticSeverity,
-    Range,
-    Diagnostic,
-    Position,
-    FoldingRange,
-    FoldingRangeParams,
-    CompletionItem,
-    CompletionList,
-    CompletionParams,
-    DiagnosticRelatedInformation,
-    Location,
-    HoverParams,
-    Hover,
-    TEXT_DOCUMENT_CODE_ACTION,
-    SemanticTokens,
-    SemanticTokensParams,
-    WillSaveTextDocumentParams,
-    TextEdit,
-    DocumentFormattingParams,
-)
-
+from debputy.analysis.debian_dir import scan_debian_dir
 from debputy.linting.lint_util import LintState
 from debputy.lsp.debputy_ls import DebputyLanguageServer
 from debputy.lsp.diagnostics import DiagnosticData
@@ -75,7 +56,6 @@ from debputy.lsp.spellchecking import default_spellchecker
 from debputy.lsp.text_util import (
     normalize_dctrl_field_name,
     LintCapablePositionCodec,
-    detect_possible_typo,
     te_range_to_lsp,
 )
 from debputy.lsp.vendoring._deb822_repro import (
@@ -86,6 +66,28 @@ from debputy.lsp.vendoring._deb822_repro.parsing import (
     Deb822KeyValuePairElement,
     LIST_SPACE_SEPARATED_INTERPRETATION,
 )
+from debputy.lsprotocol.types import (
+    DiagnosticSeverity,
+    Range,
+    Diagnostic,
+    Position,
+    FoldingRange,
+    FoldingRangeParams,
+    CompletionItem,
+    CompletionList,
+    CompletionParams,
+    DiagnosticRelatedInformation,
+    Location,
+    HoverParams,
+    Hover,
+    TEXT_DOCUMENT_CODE_ACTION,
+    SemanticTokens,
+    SemanticTokensParams,
+    WillSaveTextDocumentParams,
+    TextEdit,
+    DocumentFormattingParams,
+)
+from debputy.util import detect_possible_typo
 
 try:
     from debputy.lsp.vendoring._deb822_repro.locatable import (
@@ -94,7 +96,6 @@ try:
         START_POSITION,
     )
 
-    from pygls.server import LanguageServer
     from pygls.workspace import TextDocument
 except ImportError:
     pass
@@ -919,6 +920,7 @@ def _lint_debian_control(
 
     paragraphs = list(deb822_file)
     source_paragraph = paragraphs[0] if paragraphs else None
+    binary_stanzas_w_pos = []
 
     for paragraph_no, paragraph in enumerate(paragraphs, start=1):
         paragraph_pos = paragraph.position_in_file()
@@ -928,6 +930,7 @@ def _lint_debian_control(
         if is_binary_paragraph:
             known_fields = BINARY_FIELDS
             other_known_fields = SOURCE_FIELDS
+            binary_stanzas_w_pos.append((paragraph, paragraph_pos))
         else:
             known_fields = SOURCE_FIELDS
             other_known_fields = BINARY_FIELDS
@@ -944,9 +947,113 @@ def _lint_debian_control(
             diagnostics,
         )
 
+    _detect_misspelled_packaging_files(
+        lint_state,
+        binary_stanzas_w_pos,
+        diagnostics,
+    )
+
     return diagnostics
 
 
+def _package_range_of_stanza(
+    lint_state: LintState,
+    binary_stanzas: List[Tuple[Deb822ParagraphElement, TEPosition]],
+) -> Iterable[Tuple[str, Range]]:
+    for stanza, stanza_position in binary_stanzas:
+        kvpair = stanza.get_kvpair_element("Package")
+        if kvpair is None:
+            continue
+        representation_field_range = kvpair.range_in_parent().relative_to(
+            stanza_position
+        )
+        representation_field_range = lint_state.position_codec.range_to_client_units(
+            lint_state.lines,
+            te_range_to_lsp(representation_field_range),
+        )
+        yield stanza["Package"], representation_field_range
+
+
+def _detect_misspelled_packaging_files(
+    lint_state: LintState,
+    binary_stanzas_w_pos: List[Tuple[Deb822ParagraphElement, TEPosition]],
+    diagnostics: List[Diagnostic],
+) -> None:
+    debian_dir = lint_state.debian_dir
+    binary_packages = lint_state.binary_packages
+    if debian_dir is None or binary_packages is None:
+        return
+    all_pkg_file_data, _, _, _ = scan_debian_dir(
+        lint_state.plugin_feature_set,
+        binary_packages,
+        debian_dir,
+    )
+    stanza_ranges = {
+        p: r for p, r in _package_range_of_stanza(lint_state, binary_stanzas_w_pos)
+    }
+
+    for pkg_file_data in all_pkg_file_data:
+        binary_package = pkg_file_data.get("binary-package")
+        explicit_package = pkg_file_data.get("pkgfile-explicit-package-name", True)
+        name_segment = pkg_file_data.get("pkgfile-name-segment")
+        stem = pkg_file_data.get("pkgfile-stem")
+        if binary_package is None or stem is None:
+            continue
+        diag_range = stanza_ranges.get(binary_package)
+        if diag_range is None:
+            continue
+
+        path = pkg_file_data["path"]
+        likely_typo_of = pkg_file_data.get("likely-typo-of")
+        if likely_typo_of is not None:
+            diagnostics.append(
+                Diagnostic(
+                    diag_range,
+                    f'The file "{path}" is likely a typo of "{likely_typo_of}"',
+                    severity=DiagnosticSeverity.Warning,
+                    source="debputy",
+                    data=DiagnosticData(
+                        report_for_related_file=path,
+                    ),
+                )
+            )
+            continue
+
+        if not pkg_file_data.get("pkgfile-is-active-in-build", True):
+            diagnostics.append(
+                Diagnostic(
+                    diag_range,
+                    f"The file {path} is related to a command that is not active in the dh sequence"
+                    " with the current addons",
+                    severity=DiagnosticSeverity.Warning,
+                    source="debputy",
+                    data=DiagnosticData(
+                        report_for_related_file=path,
+                    ),
+                )
+            )
+            continue
+
+        if not explicit_package and name_segment is not None:
+            basename = os.path.basename(path)
+            alt_name = f"{binary_package}.{stem}"
+            arch_restriction = pkg_file_data.get("pkgfile-architecture-restriction")
+            if arch_restriction is not None:
+                alt_name = f"{alt_name}.{arch_restriction}"
+            diagnostics.append(
+                Diagnostic(
+                    diag_range,
+                    f'Possible typo in "{path}". Consider renaming the file to "debian/{binary_package}.{basename}"'
+                    f' or "debian/{alt_name} if it is intended for {binary_package}',
+                    severity=DiagnosticSeverity.Warning,
+                    source="debputy",
+                    data=DiagnosticData(
+                        report_for_related_file=path,
+                    ),
+                )
+            )
+
+
 @lsp_will_save_wait_until(_LANGUAGE_IDS)
 def _debian_control_on_save_formatting(
     ls: "DebputyLanguageServer",
