@@ -11,7 +11,7 @@ from typing import (
     get_origin,
 )
 
-from lsprotocol.types import (
+from debputy.lsprotocol.types import (
     Diagnostic,
     TEXT_DOCUMENT_WILL_SAVE_WAIT_UNTIL,
     Position,
@@ -46,7 +46,6 @@ from debputy.lsp.lsp_generic_yaml import (
 from debputy.lsp.quickfixes import propose_correct_text_quick_fix
 from debputy.lsp.text_util import (
     LintCapablePositionCodec,
-    detect_possible_typo,
 )
 from debputy.manifest_parser.base_types import DebputyDispatchableType
 from debputy.manifest_parser.declarative_parser import (
@@ -66,7 +65,7 @@ from debputy.plugin.api.impl_types import (
     InPackageContextParser,
     DeclarativeValuelessKeywordInputParser,
 )
-from debputy.util import _info
+from debputy.util import _info, detect_possible_typo
 from debputy.yaml.compat import (
     Node,
     CommentedMap,
