@@ -9,7 +9,7 @@ from typing import (
     Dict,
 )
 
-from lsprotocol.types import (
+from debputy.lsprotocol.types import (
     DiagnosticSeverity,
     Range,
     Diagnostic,
@@ -65,7 +65,6 @@ from debputy.lsp.spellchecking import default_spellchecker
 from debputy.lsp.text_util import (
     normalize_dctrl_field_name,
     LintCapablePositionCodec,
-    detect_possible_typo,
     te_range_to_lsp,
 )
 from debputy.lsp.vendoring._deb822_repro import (
@@ -76,6 +75,7 @@ from debputy.lsp.vendoring._deb822_repro.parsing import (
     Deb822KeyValuePairElement,
     LIST_SPACE_SEPARATED_INTERPRETATION,
 )
+from debputy.util import detect_possible_typo
 
 try:
     from debputy.lsp.vendoring._deb822_repro.locatable import (
