@@ -15,7 +15,7 @@ from typing import (
     cast,
 )
 
-from lsprotocol.types import (
+from debputy.lsprotocol.types import (
     CompletionParams,
     CompletionList,
     CompletionItem,
@@ -272,6 +272,9 @@ def deb822_completer(
     )
 
     items: Optional[Sequence[CompletionItem]]
+    markdown_kind = ls.completion_item_document_markup(
+        MarkupKind.Markdown, MarkupKind.PlainText
+    )
     if in_value:
         _info(f"Completion for field value {current_field} -- {word_at_position}")
         if known_field is None:
@@ -281,15 +284,16 @@ def deb822_completer(
             lint_state,
             list(matched_stanzas),
             value_being_completed,
+            markdown_kind,
         )
     else:
         _info("Completing field name")
         assert stanza_metadata is not None
         items = _complete_field_name(
-            ls,
             lint_state,
             stanza_metadata,
             matched_stanzas,
+            markdown_kind,
         )
 
     _info(
@@ -638,15 +642,12 @@ def deb822_semantic_tokens_full(
 
 
 def _complete_field_name(
-    ls: "DebputyLanguageServer",
     lint_state: LintState,
     fields: StanzaMetadata[Any],
     matched_stanzas: Iterable[Deb822ParagraphElement],
+    markdown_kind: MarkupKind,
 ) -> Sequence[CompletionItem]:
     items = []
-    markdown_kind = ls.completion_item_document_markup(
-        MarkupKind.Markdown, MarkupKind.PlainText
-    )
     matched_stanzas = list(matched_stanzas)
     # TODO: Normalize fields according to file rules (X[BCS]- should be stripped in some files)
     seen_fields = set(
