@@ -1466,8 +1466,7 @@ def find_json_plugin(
 def find_related_implementation_files_for_plugin(
     plugin_metadata: DebputyPluginMetadata,
 ) -> List[str]:
-    plugin_path = plugin_metadata.plugin_path
-    if not os.path.isfile(plugin_path):
+    if plugin_metadata.is_bundled:
         plugin_name = plugin_metadata.plugin_name
         _error(
             f"Cannot run find related files for {plugin_name}: The plugin seems to be bundled"
@@ -1500,7 +1499,7 @@ def find_tests_for_plugin(
     plugin_name = plugin_metadata.plugin_name
     plugin_path = plugin_metadata.plugin_path
 
-    if not os.path.isfile(plugin_path):
+    if plugin_metadata.is_bundled:
         _error(
             f"Cannot run tests for {plugin_name}: The plugin seems to be bundled or loaded via a"
             " mechanism that does not support detecting its tests."
