@@ -1,21 +1,14 @@
 import textwrap
 
+from debputy.lsprotocol.types import (
+    CompletionParams,
+    TextDocumentIdentifier,
+)
 from lsp_tests.lsp_tutil import put_doc_with_cursor
 
 try:
     from pygls.server import LanguageServer
-    from lsprotocol.types import (
-        InitializeParams,
-        ClientCapabilities,
-        GeneralClientCapabilities,
-        PositionEncodingKind,
-        TextDocumentItem,
-        Position,
-        CompletionParams,
-        TextDocumentIdentifier,
-        HoverParams,
-        MarkupContent,
-    )
+
     from debputy.lsp.lsp_debian_debputy_manifest import debputy_manifest_completer
     from debputy.lsp.debputy_ls import DebputyLanguageServer
 
