@@ -119,8 +119,10 @@ def _assert_unsupported_feature(
 
 
 def _write_manifest(manifest: HighLevelManifest) -> str:
+    mutable_manifest = manifest.mutable_manifest
+    assert mutable_manifest is not None
     with io.StringIO() as fd:
-        manifest.mutable_manifest.write_to(fd)
+        mutable_manifest.write_to(fd)
         return fd.getvalue()
 
 
@@ -134,7 +136,7 @@ def _verify_migrator_generates_parsable_manifest(
     expected_warnings: Optional[List[str]] = None,
     expected_renamed_paths: Optional[List[Tuple[str, str]]] = None,
     expected_removals: Optional[List[str]] = None,
-    required_plugins: Optional[Sequence[str]] = tuple(),
+    required_plugins: Sequence[str] = tuple(),
     dh_config_mode: Optional[int] = None,
 ) -> None:
     # No file, no changes
